"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TextParticles = void 0;
class TextParticles {
    constructor(canvas, options) {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m, _o, _p, _q;
        this.particles = [];
        this.mouse = {
            x: null,
            y: null,
            isClicked: false,
        };
        this.defaultOptions = {
            TEXT: "Text Particles",
            FONT: {
                STYLE: "bold",
                SIZE: 50,
                FAMILY: "Arial, sans-serif",
            },
            COLOR: "#ff4f4f",
            PARTICLE_SIZE_MIN: 1,
            PARTICLE_SIZE_MAX: 3,
            HOVER_RADIUS: 30,
            CLICK_RADIUS: 100,
            REPULSION_STRENGTH: 30,
            RETURN_SPEED: 10,
            INTERACTION_MODE: "hover",
        };
        this.canvas = canvas;
        this.ctx = canvas.getContext("2d");
        this.OPTIONS = Object.assign(Object.assign({}, this.defaultOptions), (options && {
            TEXT: (_a = options.TEXT) !== null && _a !== void 0 ? _a : this.defaultOptions.TEXT,
            FONT: {
                STYLE: (_c = (_b = options.FONT) === null || _b === void 0 ? void 0 : _b.STYLE) !== null && _c !== void 0 ? _c : this.defaultOptions.FONT.STYLE,
                SIZE: (_e = (_d = options.FONT) === null || _d === void 0 ? void 0 : _d.SIZE) !== null && _e !== void 0 ? _e : this.defaultOptions.FONT.SIZE,
                FAMILY: (_g = (_f = options.FONT) === null || _f === void 0 ? void 0 : _f.FAMILY) !== null && _g !== void 0 ? _g : this.defaultOptions.FONT.FAMILY,
            },
            COLOR: (_h = options.COLOR) !== null && _h !== void 0 ? _h : this.defaultOptions.COLOR,
            PARTICLE_SIZE_MIN: (_j = options.PARTICLE_SIZE_MIN) !== null && _j !== void 0 ? _j : this.defaultOptions.PARTICLE_SIZE_MIN,
            PARTICLE_SIZE_MAX: (_k = options.PARTICLE_SIZE_MAX) !== null && _k !== void 0 ? _k : this.defaultOptions.PARTICLE_SIZE_MAX,
            HOVER_RADIUS: (_l = options.HOVER_RADIUS) !== null && _l !== void 0 ? _l : this.defaultOptions.HOVER_RADIUS,
            CLICK_RADIUS: (_m = options.CLICK_RADIUS) !== null && _m !== void 0 ? _m : this.defaultOptions.CLICK_RADIUS,
            REPULSION_STRENGTH: (_o = options.REPULSION_STRENGTH) !== null && _o !== void 0 ? _o : this.defaultOptions.REPULSION_STRENGTH,
            RETURN_SPEED: (_p = options.RETURN_SPEED) !== null && _p !== void 0 ? _p : this.defaultOptions.RETURN_SPEED,
            INTERACTION_MODE: (_q = options.INTERACTION_MODE) !== null && _q !== void 0 ? _q : this.defaultOptions.INTERACTION_MODE,
        }));
        this.handleCanvasSize();
        this.init();
        this.animate();
        this.setupEventListeners();
    }
    handleCanvasSize() {
        if (this.canvas.parentElement) {
            this.canvas.width = this.canvas.parentElement.offsetWidth;
            this.canvas.height = this.canvas.parentElement.offsetHeight;
        }
    }
    init() {
        this.handleCanvasSize();
        this.particles.length = 0;
        this.ctx.clearRect(0, 0, this.canvas.width, this.canvas.height);
        const fontString = `${this.OPTIONS.FONT.STYLE} ${this.OPTIONS.FONT.SIZE}px ${this.OPTIONS.FONT.FAMILY}`;
        this.ctx.font = fontString;
        this.ctx.textAlign = "center";
        this.ctx.fillText(this.OPTIONS.TEXT, this.canvas.width / 2, this.canvas.height / 2);
        const textCoordinates = this.ctx.getImageData(0, 0, this.canvas.width, this.canvas.height);
        for (let y = 0; y < textCoordinates.height; y += 3) {
            for (let x = 0; x < textCoordinates.width; x += 3) {
                if (textCoordinates.data[y * 4 * textCoordinates.width + x * 4 + 3] > 128) {
                    this.particles.push(new Particle(x, y, this.OPTIONS, this.mouse));
                }
            }
        }
    }
    animate() {
        this.ctx.clearRect(0, 0, this.canvas.width, this.canvas.height);
        this.particles.forEach((particle) => {
            particle.draw(this.ctx);
            particle.update();
        });
        requestAnimationFrame(() => this.animate());
    }
    setupEventListeners() {
        window.addEventListener("resize", () => {
            this.handleCanvasSize();
            this.init();
        });
        window.addEventListener("mousemove", (event) => {
            this.mouse.x = event.clientX - this.canvas.getBoundingClientRect().left;
            this.mouse.y = event.clientY - this.canvas.getBoundingClientRect().top;
        });
        window.addEventListener("mousedown", () => {
            this.mouse.isClicked = true;
        });
        window.addEventListener("mouseup", () => {
            this.mouse.isClicked = false;
        });
        window.addEventListener("touchstart", (event) => {
            this.mouse.x =
                event.touches[0].clientX - this.canvas.getBoundingClientRect().left;
            this.mouse.y =
                event.touches[0].clientY - this.canvas.getBoundingClientRect().top;
            this.mouse.isClicked = true;
        });
        window.addEventListener("touchmove", (event) => {
            this.mouse.x =
                event.touches[0].clientX - this.canvas.getBoundingClientRect().left;
            this.mouse.y =
                event.touches[0].clientY - this.canvas.getBoundingClientRect().top;
        });
        window.addEventListener("touchend", () => {
            this.mouse.isClicked = false;
        });
    }
}
exports.TextParticles = TextParticles;
class Particle {
    constructor(x, y, OPTIONS, mouse) {
        this.OPTIONS = OPTIONS;
        this.mouse = mouse;
        this.x = x + (Math.random() - 0.5) * 3;
        this.y = y + (Math.random() - 0.5) * 3;
        this.size =
            Math.random() * (OPTIONS.PARTICLE_SIZE_MAX - OPTIONS.PARTICLE_SIZE_MIN) +
                OPTIONS.PARTICLE_SIZE_MIN;
        this.baseX = x;
        this.baseY = y;
        this.density = Math.random() * OPTIONS.REPULSION_STRENGTH + 1;
        this.angle = Math.random() * 360;
    }
    draw(ctx) {
        ctx.beginPath();
        ctx.arc(this.x, this.y, this.size, 0, Math.PI * 2);
        ctx.closePath();
        ctx.fillStyle = this.OPTIONS.COLOR;
        ctx.fill();
    }
    update() {
        let dx = this.mouse.x - this.x;
        let dy = this.mouse.y - this.y;
        let distance = Math.sqrt(dx * dx + dy * dy);
        let forceDirectionX = dx / distance;
        let forceDirectionY = dy / distance;
        let maxDistance = this.OPTIONS.INTERACTION_MODE === "hover"
            ? this.OPTIONS.HOVER_RADIUS
            : this.OPTIONS.CLICK_RADIUS;
        let force = (maxDistance - distance) / maxDistance;
        let directionX = forceDirectionX * force * this.density;
        let directionY = forceDirectionY * force * this.density;
        if (distance < maxDistance &&
            (this.OPTIONS.INTERACTION_MODE === "hover" ||
                (this.OPTIONS.INTERACTION_MODE === "click" && this.mouse.isClicked))) {
            this.x -= directionX;
            this.y -= directionY;
        }
        else {
            if (this.x !== this.baseX) {
                let dx = this.x - this.baseX;
                this.x -= dx / this.OPTIONS.RETURN_SPEED;
            }
            if (this.y !== this.baseY) {
                let dy = this.y - this.baseY;
                this.y -= dy / this.OPTIONS.RETURN_SPEED;
            }
        }
        this.angle += 1;
        if (this.angle >= 360)
            this.angle = 0;
    }
}
exports.default = TextParticles;
